@@ -1,7 +1,7 @@
 # constants.py
 #
 # This file is part of scqubits: a Python package for superconducting qubits,
-# arXiv:2107.08552 (2021). https://arxiv.org/abs/2107.08552
+# Quantum 5, 583 (2021). https://quantum-journal.org/papers/q-2021-11-17-583/
 #
 #    Copyright (c) 2019 and later, Jens Koch and Peter Groszkowski
 #    All rights reserved.
@@ -26,10 +26,10 @@ MODE_FUNC_DICT = {
 # the following string manipulations are used in automatic generation of default
 # ylabels of wavefunction plots
 MODE_STR_DICT = {
-    "abs_sqr": (lambda x: "$|$" + x + "$|^2$"),
-    "abs": (lambda x: "$|$" + x + "$|$"),
-    "real": (lambda x: "Re " + x),
-    "imag": (lambda x: "Im " + x),
+    "abs_sqr": (lambda x: f"$|${x}$|^2$"),
+    "abs": (lambda x: f"$|${x}$|$"),
+    "real": (lambda x: f"Re {x}"),
+    "imag": (lambda x: f"Im {x}"),
 }
 
 # enumerate variables for zero-pi qubit
