@@ -1,6 +1,7 @@
 # cos2phi_qubit.py
 #
-# This file is part of scqubits.
+# This file is part of scqubits: a Python package for superconducting qubits,
+# Quantum 5, 583 (2021). https://quantum-journal.org/papers/q-2021-11-17-583/
 #
 #    Copyright (c) 2019, Jens Koch and Peter Groszkowski
 #    All rights reserved.
@@ -22,8 +23,7 @@ from matplotlib.axes import Axes
 from matplotlib.figure import Figure
 from numpy import ndarray
 from scipy import sparse
-from scipy.sparse.csc import csc_matrix
-from scipy.sparse.dia import dia_matrix
+from scipy.sparse import csc_matrix, dia_matrix
 
 import scqubits.core.constants as constants
 import scqubits.core.descriptors as descriptors
@@ -36,7 +36,7 @@ import scqubits.core.units as units
 import scqubits.io_utils.fileio_serializers as serializers
 import scqubits.settings as settings
 import scqubits.utils.plotting as plot
-import scqubits.utils.spectrum_utils as spec_utils
+import scqubits.utils.spectrum_utils as utils
 
 from scqubits.core.noise import NOISE_PARAMS, NoisySystem, calc_therm_ratio
 from scqubits.core.storage import WaveFunctionOnGrid
@@ -65,7 +65,7 @@ class NoisyCos2PhiQubit(NoisySystem, ABC):
         pass
 
     def t1_inductive(
-        self: "Cos2PhiQubit",
+        self,
         i: int = 1,
         j: int = 0,
         Q_ind: Union[float, Callable] = None,
@@ -73,7 +73,6 @@ class NoisyCos2PhiQubit(NoisySystem, ABC):
         total: bool = True,
         esys: Tuple[ndarray, ndarray] = None,
         get_rate: bool = False,
-        **kwargs
     ) -> float:
         r"""
         :math:`T_1` due to inductive dissipation in superinductors.
@@ -129,7 +128,6 @@ class NoisyCos2PhiQubit(NoisySystem, ABC):
 
         elif callable(Q_ind):  # Q_ind is a function of omega
             q_ind_fun = Q_ind
-
         else:  # Q_ind is given as a number
 
             def q_ind_fun(omega):
@@ -150,7 +148,7 @@ class NoisyCos2PhiQubit(NoisySystem, ABC):
             )  # We assume that system energies are given in units of frequency
             return s
 
-        noise_op1 = self.phi_1_operator()  # type: ignore
+        noise_op1 = self.phi_1_operator()
 
         def spectral_density2(omega):
             therm_ratio = calc_therm_ratio(omega, T)
@@ -167,7 +165,7 @@ class NoisyCos2PhiQubit(NoisySystem, ABC):
             )  # We assume that system energies are given in units of frequency
             return s
 
-        noise_op2 = self.phi_2_operator()  # type: ignore
+        noise_op2 = self.phi_2_operator()
 
         rate_1 = self.t1(
             i=i,
@@ -177,7 +175,6 @@ class NoisyCos2PhiQubit(NoisySystem, ABC):
             total=total,
             esys=esys,
             get_rate=True,
-            **kwargs
         )
         rate_2 = self.t1(
             i=i,
@@ -187,7 +184,6 @@ class NoisyCos2PhiQubit(NoisySystem, ABC):
             total=total,
             esys=esys,
             get_rate=True,
-            **kwargs
         )
 
         if get_rate:
@@ -196,7 +192,7 @@ class NoisyCos2PhiQubit(NoisySystem, ABC):
             return 1 / (rate_1 + rate_2)
 
     def t1_capacitive(
-        self: "Cos2PhiQubit",
+        self,
         i: int = 1,
         j: int = 0,
         Q_cap: Union[float, Callable] = None,
@@ -204,7 +200,6 @@ class NoisyCos2PhiQubit(NoisySystem, ABC):
         total: bool = True,
         esys: Tuple[ndarray, ndarray] = None,
         get_rate: bool = False,
-        **kwargs
     ) -> float:
         r"""
         :math:`T_1` due to dielectric dissipation in Josephson junction
@@ -288,8 +283,8 @@ class NoisyCos2PhiQubit(NoisySystem, ABC):
             )  # We assume that system energies are given in units of frequency
             return s2
 
-        noise_op1 = self.n_1_operator()  # type: ignore
-        noise_op2 = self.n_2_operator()  # type: ignore
+        noise_op1 = self.n_1_operator()
+        noise_op2 = self.n_2_operator()
 
         rate_1 = self.t1(
             i=i,
@@ -299,7 +294,6 @@ class NoisyCos2PhiQubit(NoisySystem, ABC):
             total=total,
             esys=esys,
             get_rate=True,
-            **kwargs
         )
         rate_2 = self.t1(
             i=i,
@@ -309,7 +303,6 @@ class NoisyCos2PhiQubit(NoisySystem, ABC):
             total=total,
             esys=esys,
             get_rate=True,
-            **kwargs
         )
 
         if get_rate:
@@ -318,7 +311,7 @@ class NoisyCos2PhiQubit(NoisySystem, ABC):
             return 1 / (rate_1 + rate_2)
 
     def t1_purcell(
-        self: "Cos2PhiQubit",
+        self,
         i: int = 1,
         j: int = 0,
         Q_cap: Union[float, Callable] = None,
@@ -326,7 +319,6 @@ class NoisyCos2PhiQubit(NoisySystem, ABC):
         total: bool = True,
         esys: Tuple[ndarray, ndarray] = None,
         get_rate: bool = False,
-        **kwargs
     ) -> float:
         r"""
         :math:`T_1` due to dielectric dissipation in the shunt capacitor.
@@ -392,7 +384,7 @@ class NoisyCos2PhiQubit(NoisySystem, ABC):
             )  # We assume that system energies are given in units of frequency
             return s
 
-        noise_op = self.n_zeta_operator()  # type: ignore
+        noise_op = self.n_zeta_operator()
 
         return self.t1(
             i=i,
@@ -402,7 +394,6 @@ class NoisyCos2PhiQubit(NoisySystem, ABC):
             total=total,
             esys=esys,
             get_rate=get_rate,
-            **kwargs
         )
 
 
@@ -440,8 +431,7 @@ class Cos2PhiQubit(base.QubitBaseClass, serializers.Serializable, NoisyCos2PhiQu
     dEJ:
         disorder in junction energy
     flux:
-        external magnetic flux in angular units, 1 corresponds to one flux
-        quantum
+        external magnetic flux in units of one flux quantum
     ng:
         offset charge
     ncut:
@@ -456,18 +446,18 @@ class Cos2PhiQubit(base.QubitBaseClass, serializers.Serializable, NoisyCos2PhiQu
         optional string by which this instance can be referred to in `HilbertSpace`
         and `ParameterSweep`. If not provided, an id is auto-generated.
     """
-    EJ = descriptors.WatchedProperty("QUANTUMSYSTEM_UPDATE")
-    ECJ = descriptors.WatchedProperty("QUANTUMSYSTEM_UPDATE")
-    EL = descriptors.WatchedProperty("QUANTUMSYSTEM_UPDATE")
-    EC = descriptors.WatchedProperty("QUANTUMSYSTEM_UPDATE")
-    dCJ = descriptors.WatchedProperty("QUANTUMSYSTEM_UPDATE")
-    dL = descriptors.WatchedProperty("QUANTUMSYSTEM_UPDATE")
-    dEJ = descriptors.WatchedProperty("QUANTUMSYSTEM_UPDATE")
-    flux = descriptors.WatchedProperty("QUANTUMSYSTEM_UPDATE")
-    ng = descriptors.WatchedProperty("QUANTUMSYSTEM_UPDATE")
-    ncut = descriptors.WatchedProperty("QUANTUMSYSTEM_UPDATE")
-    zeta_cut = descriptors.WatchedProperty("QUANTUMSYSTEM_UPDATE")
-    phi_cut = descriptors.WatchedProperty("QUANTUMSYSTEM_UPDATE")
+    EJ = descriptors.WatchedProperty(float, "QUANTUMSYSTEM_UPDATE")
+    ECJ = descriptors.WatchedProperty(float, "QUANTUMSYSTEM_UPDATE")
+    EL = descriptors.WatchedProperty(float, "QUANTUMSYSTEM_UPDATE")
+    EC = descriptors.WatchedProperty(float, "QUANTUMSYSTEM_UPDATE")
+    dCJ = descriptors.WatchedProperty(float, "QUANTUMSYSTEM_UPDATE")
+    dL = descriptors.WatchedProperty(float, "QUANTUMSYSTEM_UPDATE")
+    dEJ = descriptors.WatchedProperty(float, "QUANTUMSYSTEM_UPDATE")
+    flux = descriptors.WatchedProperty(float, "QUANTUMSYSTEM_UPDATE")
+    ng = descriptors.WatchedProperty(float, "QUANTUMSYSTEM_UPDATE")
+    ncut = descriptors.WatchedProperty(int, "QUANTUMSYSTEM_UPDATE")
+    zeta_cut = descriptors.WatchedProperty(int, "QUANTUMSYSTEM_UPDATE")
+    phi_cut = descriptors.WatchedProperty(int, "QUANTUMSYSTEM_UPDATE")
 
     def __init__(
         self,
@@ -500,7 +490,6 @@ class Cos2PhiQubit(base.QubitBaseClass, serializers.Serializable, NoisyCos2PhiQu
         self.zeta_cut = zeta_cut
         self.phi_cut = phi_cut
         self.truncated_dim = truncated_dim
-        self._evec_dtype = np.float_
         self._default_phi_grid = discretization.Grid1d(-4 * np.pi, 4 * np.pi, 100)
         self._default_zeta_grid = discretization.Grid1d(-4 * np.pi, 4 * np.pi, 100)
         self._default_theta_grid = discretization.Grid1d(-0.5 * np.pi, 1.5 * np.pi, 100)
@@ -570,14 +559,14 @@ class Cos2PhiQubit(base.QubitBaseClass, serializers.Serializable, NoisyCos2PhiQu
         Returns
         -------
             inductive energy renormalized by with disorder"""
-        return self.EL / (1 - self.dL ** 2)
+        return self.EL / (1 - self.dL**2)
 
     def _disordered_ecj(self) -> float:
         """
         Returns
         -------
             junction capacitance energy renormalized by with disorder"""
-        return self.ECJ / (1 - self.dCJ ** 2)
+        return self.ECJ / (1 - self.dCJ**2)
 
     def phi_osc(self) -> float:
         """
@@ -866,27 +855,25 @@ class Cos2PhiQubit(base.QubitBaseClass, serializers.Serializable, NoisyCos2PhiQu
 
     def _evals_calc(self, evals_count) -> ndarray:
         hamiltonian_mat = self.hamiltonian()
-        evals = sparse.linalg.eigsh(
+        evals = utils.eigsh_safe(
             hamiltonian_mat,
             k=evals_count,
             return_eigenvectors=False,
             sigma=0.0,
             which="LM",
-            v0=settings.RANDOM_ARRAY[: self.hilbertdim()],
         )
         return np.sort(evals)
 
     def _esys_calc(self, evals_count) -> Tuple[ndarray, ndarray]:
         hamiltonian_mat = self.hamiltonian()
-        evals, evecs = sparse.linalg.eigsh(
+        evals, evecs = utils.eigsh_safe(
             hamiltonian_mat,
             k=evals_count,
             return_eigenvectors=True,
             sigma=0.0,
             which="LM",
-            v0=settings.RANDOM_ARRAY[: self.hilbertdim()],
         )
-        evals, evecs = spec_utils.order_eigensystem(evals, evecs)
+        evals, evecs = utils.order_eigensystem(evals, evecs)
         return evals, evecs
 
     def potential(self, phi, zeta, theta) -> float:
@@ -1076,7 +1063,7 @@ class Cos2PhiQubit(base.QubitBaseClass, serializers.Serializable, NoisyCos2PhiQu
         )
         wavefunc.amplitudes = np.transpose(
             amplitude_modifier(
-                spec_utils.standardize_phases(
+                utils.standardize_phases(
                     wavefunc.amplitudes.reshape(phi_grid.pt_count, theta_grid.pt_count)
                 )
             )
