@@ -1,6 +1,7 @@
 # fileio_serializers.py
 #
-# This file is part of scqubits.
+# This file is part of scqubits: a Python package for superconducting qubits,
+# Quantum 5, 583 (2021). https://quantum-journal.org/papers/q-2021-11-17-583/
 #
 #    Copyright (c) 2019 and later, Jens Koch and Peter Groszkowski
 #    All rights reserved.
@@ -14,7 +15,7 @@ Helper classes for writing data to files.
 
 import inspect
 
-from abc import ABC, ABCMeta
+from abc import ABCMeta
 from collections import OrderedDict
 from numbers import Number
 from typing import TYPE_CHECKING, Any, Callable, Dict, List, Tuple, Union
@@ -23,6 +24,8 @@ import numpy as np
 
 from numpy import ndarray
 from scipy.sparse import csc_matrix
+from sympy import Expr
+from typing_extensions import Protocol, runtime_checkable
 
 import scqubits.utils.misc as utils
 
@@ -33,7 +36,8 @@ if TYPE_CHECKING:
 SERIALIZABLE_REGISTRY = {}
 
 
-class Serializable(ABC):
+@runtime_checkable
+class Serializable(Protocol):
     """Mix-in class that makes descendant classes serializable."""
 
     _subclasses: List[ABCMeta] = []
@@ -45,8 +49,8 @@ class Serializable(ABC):
         return super().__new__(cls)
 
     def __init_subclass__(cls) -> None:
-        """Used to register all non-abstract subclasses as a list in
-        `QuantumSystem.subclasses`."""
+        """Used to register all non-abstract _subclasses as a list in
+        `QuantumSystem._subclasses`."""
         super().__init_subclass__()
         if not inspect.isabstract(cls):
             cls._subclasses.append(cls)
@@ -66,7 +70,7 @@ class Serializable(ABC):
         """
         initdata = {name: getattr(self, name) for name in self._init_params}
         if hasattr(self, "_id_str"):
-            initdata["id_str"] = self._id_str
+            initdata["id_str"] = self._id_str  # type:ignore
         iodata = dict_serialize(initdata)
         iodata.typename = type(self).__name__
         return iodata
@@ -126,7 +130,7 @@ def _add_attribute(
     return attributes, ndarrays, objects
 
 
-TO_ATTRIBUTE = (str, Number, dict, list, tuple, bool, np.bool_)
+TO_ATTRIBUTE = (Expr, str, Number, dict, OrderedDict, list, tuple, bool, np.bool_)
 TO_NDARRAY = (np.ndarray,)
 TO_OBJECT = (Serializable,)
 
@@ -148,6 +152,24 @@ def type_dispatch(entity: Serializable) -> Callable:
     return _add_object
 
 
+def Expr_serialize(expr_instance: Expr) -> "IOData":
+    """
+    Create an IODate instance for a sympy expression via string conversion
+    """
+    import scqubits.io_utils.fileio as io
+
+    attributes: Dict[str, Any] = {}
+    ndarrays: Dict[str, ndarray] = {}
+    objects: Dict[str, object] = {}
+    typename = "Expr"
+    item = str(expr_instance)
+    update_func = type_dispatch(item)
+    attributes, ndarrays, objects = update_func(
+        "Expr", item, attributes, ndarrays, objects
+    )
+    return io.IOData(typename, attributes, ndarrays, objects)
+
+
 def dict_serialize(dict_instance: Dict[str, Any]) -> "IOData":
     """
     Create an IOData instance from dictionary data.
@@ -175,15 +197,17 @@ def OrderedDict_serialize(dict_instance: Dict[str, Any]) -> "IOData":
     import scqubits.io_utils.fileio as io
 
     dict_instance = utils.remove_nones(dict_instance)
+
     attributes: Dict[str, Any] = {}
     ndarrays: Dict[str, ndarray] = {}
     objects: Dict[str, object] = {}
     typename = "OrderedDict"
 
-    for name, content in dict_instance.items():
-        update_func = type_dispatch(content)
+    list_representation = list(dict_instance.items())
+    for index, item in enumerate(list_representation):
+        update_func = type_dispatch(item)
         attributes, ndarrays, objects = update_func(
-            name, content, attributes, ndarrays, objects
+            str(index), item, attributes, ndarrays, objects
         )
     return io.IOData(typename, attributes, ndarrays, objects)
 
@@ -272,6 +296,13 @@ def range_serialize(range_instance: range) -> "IOData":
     return io.IOData(typename, attributes, ndarrays, objects)
 
 
+def Expr_deserialize(iodata: "IOData") -> Expr:
+    """Turn IOData instance back into a dict"""
+    from sympy import sympify
+
+    return sympify(iodata["Expr"])
+
+
 def dict_deserialize(iodata: "IOData") -> Dict[str, Any]:
     """Turn IOData instance back into a dict"""
     return dict(**iodata.as_kwargs())
@@ -279,7 +310,8 @@ def dict_deserialize(iodata: "IOData") -> Dict[str, Any]:
 
 def OrderedDict_deserialize(iodata: "IOData") -> Dict[str, Any]:
     """Turn IOData instance back into a dict"""
-    return OrderedDict([(name, values) for name, values in iodata.as_kwargs().items()])
+    dict_data = iodata.as_kwargs()
+    return OrderedDict([dict_data[key] for key in sorted(dict_data, key=int)])
 
 
 def csc_matrix_deserialize(iodata: "IOData") -> csc_matrix:
@@ -327,6 +359,4 @@ def get_init_params(obj: Serializable) -> List[str]:
         init_params.remove("self")
     if "kwargs" in init_params:
         init_params.remove("kwargs")
-    # if "id_str" in init_params:
-    #     init_params.remove("id_str")
     return init_params
