@@ -1,6 +1,7 @@
 # spectrum_utils.py
 #
-# This file is part of scqubits.
+# This file is part of scqubits: a Python package for superconducting qubits,
+# Quantum 5, 583 (2021). https://quantum-journal.org/papers/q-2021-11-17-583/
 #
 #    Copyright (c) 2019 and later, Jens Koch and Peter Groszkowski
 #    All rights reserved.
@@ -15,17 +16,45 @@ from typing import TYPE_CHECKING, List, Optional, Tuple, Union
 
 import numpy as np
 import qutip as qt
+import scipy as sp
 
 from numpy import ndarray
 from qutip import Qobj
 from scipy.sparse import csc_matrix, dia_matrix
 
+import scqubits.settings as settings
+
 if TYPE_CHECKING:
     from scqubits import Oscillator, ParameterSweep, SpectrumData
     from scqubits.core.qubit_base import QubitBaseClass
     from scqubits.io_utils.fileio_qutip import QutipEigenstates
 
-    QuantumSys = Union[QubitBaseClass, Oscillator]
+from scqubits.utils.typedefs import QuantumSys
+
+
+def eigsh_safe(*args, **kwargs):
+    """Wrapper method for scipy.sparse.linalg.eigsh
+    This ensures:
+    1. Always use the same "random" starting vector v0. Otherwise results show
+       random behavior (small deviations between different runs, problem for pytests)
+    2. Test for degenerate eigenvalues. If there are any, need to orthogonalize the
+        eigenvectors properly."""
+    mat_size = args[0].shape[0]
+    kwargs["v0"] = settings.RANDOM_ARRAY[:mat_size]
+
+    if kwargs.get("return_eigenvectors"):
+        evals, evecs = sp.sparse.linalg.eigsh(*args, **kwargs)
+        if has_degeneracy(evals):
+            evecs, _ = sp.linalg.qr(evecs, mode="economic")
+        return evals, evecs
+    return sp.sparse.linalg.eigsh(*args, **kwargs)
+
+
+def has_degeneracy(evals: ndarray) -> bool:
+    evals_rightpad = np.pad(evals, (0, 1))
+    evals_leftpad = np.pad(evals, (1, 0))
+    evals_neighbor_diffs = evals_leftpad - evals_rightpad
+    return np.isclose(np.min(np.abs(evals_neighbor_diffs)), 0)
 
 
 def order_eigensystem(
@@ -127,18 +156,15 @@ def matrix_element(
 
     if isinstance(state1, qt.Qobj):
         vec1 = state1.data.toarray()
-        vec2 = state2.data.toarray()
     else:
         vec1 = state1
+
+    if isinstance(state2, qt.Qobj):
+        vec2 = state2.data.toarray()
+    else:
         vec2 = state2
 
-    if isinstance(op_matrix, np.ndarray):  # Is operator given in dense form?
-        return np.vdot(
-            vec1, np.dot(operator, vec2)
-        )  # Yes - use numpy's 'vdot' and 'dot'.
-    return np.vdot(
-        vec1, op_matrix.dot(vec2)
-    )  # No, operator is sparse. Must use its own 'dot' method.
+    return vec1.conj().T @ op_matrix @ vec2
 
 
 def get_matrixelement_table(
@@ -160,20 +186,12 @@ def get_matrixelement_table(
         table of matrix elements
     """
     if isinstance(operator, qt.Qobj):
-        state_list = state_table
+        states_in_columns = state_table.T
     else:
-        state_list = state_table.T
+        states_in_columns = state_table
 
-    tablesize = len(state_list)
-    mtable = [
-        [
-            matrix_element(state_list[n], operator, state_list[m])
-            for m in range(tablesize)
-        ]
-        for n in range(tablesize)
-    ]
-
-    return np.asarray(mtable)
+    mtable = states_in_columns.conj().T @ operator @ states_in_columns
+    return mtable
 
 
 def closest_dressed_energy(
@@ -242,7 +260,8 @@ def absorption_spectrum(spectrum_data: "SpectrumData") -> "SpectrumData":
     the energy of the select state. Resulting negative frequencies, if the reference
     state is not the ground state, are omitted.
     """
-    spectrum_data.energy_table = spectrum_data.energy_table.clip(min=0.0)
+    assert isinstance(spectrum_data.energy_table, ndarray)
+    spectrum_data.energy_table = spectrum_data.energy_table.clip(min=0.0)  # type:ignore
     return spectrum_data
 
 
@@ -253,8 +272,9 @@ def emission_spectrum(spectrum_data: "SpectrumData") -> "SpectrumData":
     select state, and multiplying the result by -1. Resulting negative frequencies,
     corresponding to absorption instead, are omitted.
     """
+    assert isinstance(spectrum_data.energy_table, ndarray)
     spectrum_data.energy_table *= -1.0
-    spectrum_data.energy_table = spectrum_data.energy_table.clip(min=0.0)
+    spectrum_data.energy_table = spectrum_data.energy_table.clip(min=0.0)  # type:ignore
     return spectrum_data
 
 
@@ -355,12 +375,8 @@ def generate_target_states_list(
 
 
 def recast_esys_mapdata(
-    esys_mapdata: Union[
-        List[Tuple[np.ndarray, np.ndarray]], List[Tuple[np.ndarray, "QutipEigenstates"]]
-    ]
-) -> Union[
-    Tuple[np.ndarray, "List[QutipEigenstates]"], Tuple[np.ndarray, List[np.ndarray]]
-]:
+    esys_mapdata: List[Tuple[np.ndarray, np.ndarray]]
+) -> Tuple[np.ndarray, List[np.ndarray]]:
     """
     Takes data generated by a map of eigensystem calls and returns the eigenvalue and
     eigenstate tables
@@ -384,8 +400,12 @@ def identity_wrap(
     op_in_eigenbasis: bool = False,
     evecs: ndarray = None,
 ) -> Qobj:
-    """Wrap given operator in subspace `subsystem` in identity operators to form full
-    Hilbert-space operator.
+    """Takes the `operator` belonging to `subsystem` and "wraps" it in identities.
+    The full Hilbert space is taken to consist of all subsystems given as
+    `subsys_list`. `subsystem` must be one element in that list. For each of the
+    other subsystems in the list, an identity operator of the correct dimension is
+    generated and inserted into the appropriate Kronecker product "sandwiching" the
+    operator.
 
     Parameters
     ----------
@@ -397,13 +417,21 @@ def identity_wrap(
     subsys_list:
         list of all subsystems relevant to the Hilbert space.
     op_in_eigenbasis:
-        whether `operator` is given in the `subsystem` eigenbasis; otherwise, the
-        internal QuantumSys basis is assumed
+        whether `operator` is given in the `subsystem` eigenbasis; otherwise,
+        `operator` is assumed to be in the internal QuantumSystem basis. This
+        argument is ignored if `operator` is given as a Qobj.
     evecs:
-        internal QuantumSys eigenstates, used to convert `operator` into eigenbasis
+        internal `QuantumSystem` eigenstates, used to convert `operator` into eigenbasis
+
+    Returns
+    -------
+        operator in the full Hilbert space (as specified by `subsystem_list`). This
+        operator is expressed in the bare product basis consisting of the energy
+        eigenstates of each subsystem (unless `operator` is provided as a `Qobj`,
+        in which case no conversion takes place).
     """
     subsys_operator = convert_operator_to_qobj(
-        operator, subsystem, op_in_eigenbasis, evecs
+        operator, subsystem, op_in_eigenbasis, evecs  # type:ignore
     )
     operator_identitywrap_list = [
         qt.operators.qeye(the_subsys.truncated_dim) for the_subsys in subsys_list
